const defaultGetValue = points => points.length;

export default class BinSorter {
  constructor(bins = [], getValue = defaultGetValue) {
    this.sortedBins = this.getSortedBins(bins, getValue);
    this.maxCount = this.getMaxCount();
    this.binMap = this.getBinMap();
  }

  getSortedBins(bins, getValue) {
    return bins.reduce((accu, h, i) => {
      const value = getValue(h.points);

      if (value !== null && value !== undefined) {
        accu.push({
          i: Number.isFinite(h.index) ? h.index : i,
          value,
          counts: h.points.length
        });
      }

      return accu;
    }, []).sort((a, b) => a.value - b.value);
  }

  getValueRange([lower, upper]) {
    const len = this.sortedBins.length;

    if (!len) {
      return [0, 0];
    }

    const lowerIdx = Math.ceil(lower / 100 * (len - 1));
    const upperIdx = Math.floor(upper / 100 * (len - 1));
    return [this.sortedBins[lowerIdx].value, this.sortedBins[upperIdx].value];
  }

  getMaxCount() {
    let maxCount = 0;
    this.sortedBins.forEach(x => maxCount = maxCount > x.counts ? maxCount : x.counts);
    return maxCount;
  }

  getBinMap() {
    return this.sortedBins.reduce((mapper, curr) => Object.assign(mapper, {
      [curr.i]: curr
    }), {});
  }

}
//# sourceMappingURL=bin-sorter.js.map